import { IAnnotationBuilder } from './IAnnotationBuilder';
import { Token, IContainer, Type } from '@ts-ioc/core';
import { AnnotationConfigure } from './AnnotationConfigure';
/**
 * Annotation class builder. build class with metadata and config.
 *
 * @export
 * @class AnnotationBuilder
 * @implements {implements IAnnotationBuilder<T>}
 * @template T
 */
export declare class AnnotationBuilder<T> implements IAnnotationBuilder<T> {
    /**
     * ioc container.
     *
     * @type {IContainer}
     * @memberof BootBuilder
     */
    container: IContainer;
    constructor();
    build(token: Token<T>, config?: AnnotationConfigure<T>, data?: any): Promise<T>;
    buildByConfig(config: Token<T> | AnnotationConfigure<T>, data?: any): Promise<any>;
    createInstance(token: Token<T>, config: AnnotationConfigure<T>, data?: any): Promise<T>;
    getBuilder(token: Token<T>, config?: AnnotationConfigure<T>): IAnnotationBuilder<T>;
    /**
     * bundle instance via config.
     *
     * @param {T} instance
     * @param {AnnotationConfigure} config
     * @param {IContainer} [container]
     * @returns {Promise<T>}
     * @memberof BootBuilder
     */
    buildStrategy(instance: T, config: AnnotationConfigure<T>): Promise<T>;
    getType(config: AnnotationConfigure<T>): Token<T>;
    protected getTokenMetaConfig(token: Token<T>, config?: AnnotationConfigure<T>): AnnotationConfigure<T>;
    getDecorator(): string;
    protected getMetaConfig(token: Type<any>): AnnotationConfigure<T>;
    protected isEqual(build: IAnnotationBuilder<T>): boolean;
    protected resolveToken(token: Token<T>, data?: any): T;
    static classAnnations: any;
}
